@@ -7,11 +7,12 @@ from .model import Channel, Message, MessageReference, AllowedMentions, Snowflak
     ThreadMember, ListThreadsResponse, Component, Role, ApplicationCommandOption, GuildApplicationCommandPermissions, \
     ApplicationCommandPermissions, VerificationLevel, DefaultMessageNotificationLevel, ExplicitContentFilterLevel, \
     SystemChannelFlags, GuildPreview, ChannelTypes, GuildMember, Ban, PermissionFlags, GuildWidget, FILE_TYPE, \
-    VoiceRegion, Integration, ApplicationCommandTypes, WelcomeScreen, WelcomeScreenChannel, PrivacyLevel, StageInstance
+    VoiceRegion, Integration, ApplicationCommandTypes, WelcomeScreen, WelcomeScreenChannel, PrivacyLevel, StageInstance, \
+    AuditLog, AuditLogEvents, GuildTemplate, BYTES_RESPONSE, Sticker, GetGateway
 from .utils import from_emoji, wrap_to_async, to_image_data
 
 if typing.TYPE_CHECKING:
-    from .base.model import AbstractObject
+    from .base.model import AbstractObject, DiscordObjectBase
 
 
 class APIClient:
@@ -41,6 +42,26 @@ class APIClient:
         self.application: typing.Optional[Application] = None
         self.application_id: typing.Optional[Snowflake] = Snowflake.ensure_snowflake(application_id)
 
+    # Audit Log
+
+    def request_guild_audit_log(self,
+                                guild: Guild.TYPING,
+                                *,
+                                user: User.TYPING = None,
+                                action_type: typing.Union[int, AuditLogEvents] = None,
+                                before: "DiscordObjectBase.TYPING" = None,
+                                limit: int = None) -> AuditLog.RESPONSE:
+        if user is not None:
+            user = str(int(user))
+        if action_type is not None:
+            action_type = int(action_type)
+        if before is not None:
+            before = str(int(before))
+        resp = self.http.request_guild_audit_log(int(guild), user, action_type, before, limit)
+        if isinstance(resp, dict):
+            return AuditLog(self, resp)
+        return wrap_to_async(AuditLog, self, resp, as_create=False)
+
     # Channel
 
     def request_channel(self, channel: Channel.TYPING) -> Channel.RESPONSE:
@@ -68,7 +89,7 @@ class APIClient:
         if permission_overwrites:
             permission_overwrites = [x.to_dict() for x in permission_overwrites]
         if parent:
-            parent = int(parent)
+            parent = int(parent)  # noqa
         channel = self.http.modify_guild_channel(int(channel), name, channel_type, position, topic, nsfw, rate_limit_per_user,
                                                  bitrate, user_limit, permission_overwrites, parent, rtc_region, video_quality_mode, reason=reason)
         if isinstance(channel, dict):
@@ -97,8 +118,8 @@ class APIClient:
     def delete_channel(self, channel: Channel.TYPING, *, reason: str = None) -> Channel.RESPONSE:
         resp = self.http.delete_channel(int(channel), reason=reason)
         if isinstance(resp, dict):
-            return Channel.create(self, resp)
-        return wrap_to_async(Channel, self, resp)
+            return Channel.create(self, resp, prevent_caching=True)
+        return wrap_to_async(Channel, self, resp, prevent_caching=True)
 
     def request_channel_messages(self,
                                  channel: Channel.TYPING, *,
@@ -130,7 +151,9 @@ class APIClient:
                        allowed_mentions: typing.Union[AllowedMentions, dict] = None,
                        message_reference: typing.Union[Message, MessageReference, dict] = None,
                        component: typing.Union[dict, Component] = None,
-                       components: typing.List[typing.Union[dict, Component]] = None) -> Message.RESPONSE:
+                       components: typing.List[typing.Union[dict, Component]] = None,
+                       sticker: Sticker.TYPING = None,
+                       stickers: typing.List[Sticker.TYPING] = None) -> Message.RESPONSE:
         """
         Sends message create request to API.
 
@@ -153,6 +176,8 @@ class APIClient:
         :param message_reference: Message to reply.
         :param component: Component of the message.
         :param components: List of  components of the message.
+        :param sticker: Sticker of the message.
+        :param stickers: Stickers of the message. Up to 3.
         :return: Union[:class:`.model.channel.Message`, Coroutine[dict]]
         """
         if files and file:
@@ -180,6 +205,12 @@ class APIClient:
             components = [component]
         if components:
             components = [*map(lambda n: n if isinstance(n, dict) else n.to_dict(), components)]
+        if sticker and stickers:
+            raise TypeError("you can't pass both sticker and stickers.")
+        if sticker:
+            stickers = [sticker]
+        if stickers:
+            stickers = [*map(int, stickers)]
         params = {"channel_id": int(channel),
                   "content": content,
                   "embeds": embeds,
@@ -187,7 +218,8 @@ class APIClient:
                   "message_reference": message_reference,
                   "tts": tts,
                   "allowed_mentions": self.get_allowed_mentions(allowed_mentions),
-                  "components": components}
+                  "components": components,
+                  "sticker_ids": stickers}
         if files:
             params["files"] = files
         try:
@@ -728,7 +760,7 @@ class APIClient:
         if deaf is not EmptyObject:
             kwargs["deaf"] = deaf
         if channel is not EmptyObject:
-            kwargs["channel_id"] = int(channel) if channel else channel
+            kwargs["channel_id"] = int(channel) if channel else channel  # noqa
         resp = self.http.modify_guild_member(int(guild), int(user), **kwargs, reason=reason)
         if isinstance(resp, dict):
             return GuildMember.create(self, resp, guild_id=int(guild))
@@ -910,7 +942,7 @@ class APIClient:
         return wrap_to_async(GuildWidget, None, resp, as_create=False)
 
     def modify_guild_widget(self, guild: Guild.TYPING, *, enabled: bool = None, channel: Channel.TYPING = EmptyObject, reason: str = None) -> GuildWidget.RESPONSE:
-        resp = self.http.modify_guild_widget(int(guild), enabled, channel, reason=reason)
+        resp = self.http.modify_guild_widget(int(guild), enabled, channel, reason=reason)  # noqa
         if isinstance(resp, dict):
             return GuildWidget(resp)
         return wrap_to_async(GuildWidget, None, resp, as_create=False)
@@ -929,7 +961,7 @@ class APIClient:
             return AbstractObject(resp)
         return wrap_to_async(AbstractObject, None, resp, as_create=False)
 
-    def request_guild_widget_image(self, guild: Guild.TYPING, style: typing.Optional[str] = None) -> bytes:
+    def request_guild_widget_image(self, guild: Guild.TYPING, style: typing.Optional[str] = None) -> BYTES_RESPONSE:
         return self.http.request_guild_widget_image(int(guild), style)
 
     def request_guild_welcome_screen(self, guild: Guild.TYPING) -> WelcomeScreen.RESPONSE:
@@ -965,6 +997,50 @@ class APIClient:
                 request_to_speak_timestamp.isoformat()
         return self.http.modify_user_voice_state(int(guild), str(int(channel)), user, suppress, request_to_speak_timestamp)
 
+    # Guild Template
+
+    def request_guild_template(self, template: GuildTemplate.TYPING) -> GuildTemplate.RESPONSE:
+        resp = self.http.request_guild_template(str(template))
+        if isinstance(resp, dict):
+            return GuildTemplate(self, resp)
+        return wrap_to_async(GuildTemplate, self, resp, as_create=False)
+
+    def create_guild_from_template(self, template: GuildTemplate.TYPING, name: str, *, icon: str = None) -> Guild.RESPONSE:
+        resp = self.http.create_guild_from_template(str(template), name, icon)
+        if isinstance(resp, dict):
+            return Guild.create(self, resp)
+        return wrap_to_async(Guild, self, resp)
+
+    def request_guild_templates(self, guild: Guild.TYPING) -> GuildTemplate.RESPONSE_AS_LIST:
+        resp = self.http.request_guild_templates(int(guild))
+        if isinstance(resp, list):
+            return [GuildTemplate(self, x) for x in resp]
+        return wrap_to_async(GuildTemplate, self, resp, as_create=False)
+
+    def create_guild_template(self, guild: Guild.TYPING, name: str, *, description: str = EmptyObject) -> GuildTemplate.RESPONSE:
+        resp = self.http.create_guild_template(int(guild), name, description)
+        if isinstance(resp, dict):
+            return GuildTemplate(self, resp)
+        return wrap_to_async(GuildTemplate, self, resp, as_create=False)
+
+    def sync_guild_template(self, guild: Guild.TYPING, template: GuildTemplate.TYPING) -> GuildTemplate.RESPONSE:
+        resp = self.http.sync_guild_template(int(guild), str(template))
+        if isinstance(resp, dict):
+            return GuildTemplate(self, resp)
+        return wrap_to_async(GuildTemplate, self, resp, as_create=False)
+
+    def modify_guild_template(self, guild: Guild.TYPING, template: GuildTemplate.TYPING, name: str = None, description: str = EmptyObject) -> GuildTemplate.RESPONSE:
+        resp = self.http.modify_guild_template(int(guild), str(template), name, description)
+        if isinstance(resp, dict):
+            return GuildTemplate(self, resp)
+        return wrap_to_async(GuildTemplate, self, resp, as_create=False)
+
+    def delete_guild_template(self, guild: Guild.TYPING, template: GuildTemplate.TYPING) -> GuildTemplate.RESPONSE:
+        resp = self.http.delete_guild_template(int(guild), str(template))
+        if isinstance(resp, dict):
+            return GuildTemplate(self, resp)
+        return wrap_to_async(GuildTemplate, self, resp, as_create=False)
+
     # Invite
 
     def request_invite(self, invite_code: typing.Union[str, Invite], *, with_counts: bool = None, with_expiration: bool = None) -> Invite.RESPONSE:
@@ -1552,6 +1628,14 @@ class APIClient:
             return Application(self, resp)
         return wrap_to_async(Application, self, resp, as_create=False)
 
+    # Gateway
+
+    def request_gateway(self, *, bot: bool = True) -> typing.Union[GetGateway, typing.Awaitable[GetGateway]]:
+        resp = self.http.request_gateway(bot)
+        if isinstance(resp, dict):
+            return GetGateway(resp)
+        return wrap_to_async(GetGateway, None, resp, as_create=False)
+
     # Misc
 
     def get_allowed_mentions(self, allowed_mentions):
