@@ -12,6 +12,7 @@ from .cache import CacheContainer
 from .exception import WebsocketClosed
 from .handler import EventHandler
 from .model import Intents, AllowedMentions, Snowflake, Activity, Guild, Channel
+from .utils import get_shard_id
 
 
 class Client(APIClient):
@@ -38,24 +39,30 @@ class Client(APIClient):
     def __init__(self,
                  token: str, *,
                  intents: Intents = Intents.no_privileged(),
-                 default_allowed_mentions: AllowedMentions = None,
-                 loop=None,
+                 default_allowed_mentions: typing.Optional[AllowedMentions] = None,
+                 loop: typing.Optional[asyncio.AbstractEventLoop] = None,
                  cache: bool = True,
-                 application_id: typing.Union[str, int, Snowflake] = None,
-                 **cache_max_sizes):
+                 application_id: Snowflake.TYPING = None,
+                 monoshard: bool = False,
+                 shard_count: typing.Optional[int] = None,
+                 **cache_max_sizes: int):
         cache_max_sizes.setdefault("message", 1000)
-        self.loop = loop or asyncio.get_event_loop()
+        self.loop: asyncio.AbstractEventLoop = loop or asyncio.get_event_loop()
         super().__init__(token, base=AsyncHTTPRequest, default_allowed_mentions=default_allowed_mentions, loop=loop)
-        self.token = token
+        self.token: str = token
         self.__use_cache = cache
-        self.cache = CacheContainer(**cache_max_sizes) if self.__use_cache else None  # The reason only supporting caching with WS is to ensure up-to-date object.
+        self.cache: typing.Optional[CacheContainer] = CacheContainer(**cache_max_sizes) if self.__use_cache else None
+        # The reason only supporting caching with WS is to ensure up-to-date object.
         self.__ws_class = WebSocketClient
-        self.intents = intents
+        self.intents: Intents = intents
         self.ws: typing.Union[None, WebSocketClient] = None
-        self.events = EventHandler(self)
+        self.events: EventHandler = EventHandler(self)
         self.__wait_futures = {}
-        self.application_id = Snowflake.ensure_snowflake(application_id)
+        self.application_id: typing.Optional[Snowflake] = Snowflake.ensure_snowflake(application_id)
         self.__ready_future = asyncio.Future()
+        self.monoshard: bool = monoshard
+        self.shard_count: typing.Optional[int] = shard_count
+        self.__shards = {} if self.monoshard else None
 
         # Internal events dispatch
         self.events.add("READY", self.__ready)
@@ -72,7 +79,7 @@ class Client(APIClient):
             if user:
                 user.set_voice_state(voice_state)
 
-    def on_(self, name: str = None, meth=None):
+    def on_(self, name: str = None, meth=None) -> typing.Any:
         """
         Adds new event listener. This can be used as decorator or function.
 
@@ -90,7 +97,7 @@ class Client(APIClient):
         """Alias of :meth:`.on_`"""
         return self.on_
 
-    def wait(self, event_name: str, timeout: float = None, check: typing.Callable[[typing.Any], bool] = None):
+    def wait(self, event_name: str, timeout: float = None, check: typing.Callable[[typing.Any], bool] = None) -> typing.Any:
         """
         Waits for the event dispatch.
 
@@ -131,6 +138,15 @@ class Client(APIClient):
                 if not fut.cancelled():
                     fut.set_result(args)
 
+    def get_shard_id(self, guild: Guild.TYPING):
+        if self.__shards:
+            return get_shard_id(int(guild), len(self.__shards))
+
+    def get_shard(self, guild: Guild.TYPING) -> typing.Optional[WebSocketClient]:
+        if self.__shards:
+            shard_id = get_shard_id(int(guild), len(self.__shards))
+            return self.__shards.get(shard_id)
+
     async def wait_ready(self):
         if not self.__ready_future.done():
             await self.__ready_future
@@ -148,15 +164,32 @@ class Client(APIClient):
         .. warning::
             You can call this only once.
         """
-        self.ws = await self.__ws_class.connect(self.http, self.intents, self.events, reconnect_on_unknown_disconnect, compress)
-        await self.ws.run()
+        if self.monoshard:
+            gateway = await self.request_gateway()
+            shard_count = self.shard_count or gateway.shards
+            if self.shard_count is None:
+                self.shard_count = gateway.shards
+            for x in range(shard_count):
+                ws = await self.__ws_class.connect_without_request(
+                    gateway, self.http, self.intents, self.events, reconnect_on_unknown_disconnect, compress, shard=[x, shard_count]
+                )
+                self.__shards[x] = ws
+                await ws.receive_once()
+                self.loop.create_task(ws.run())
+        else:
+            self.ws = await self.__ws_class.connect(self.http, self.intents, self.events, reconnect_on_unknown_disconnect, compress)
+            await self.ws.run()
 
     async def close(self):
         """Clears all connections and closes session."""
-        await self.ws.close()
+        if self.ws:
+            await self.ws.close()
+        elif self.__shards:
+            for x in self.__shards.values():
+                await x.close()
         await self.http.close()
 
-    def update_presence(self, *, since: int = None, activities: typing.List[typing.Union[Activity, dict]], status: str = "online", afk: bool = False):
+    async def update_presence(self, *, since: int = None, activities: typing.List[typing.Union[Activity, dict]], status: str = "online", afk: bool = False):
         """
         Updates the bot presence.
 
@@ -168,11 +201,15 @@ class Client(APIClient):
         :param afk: Whether the bot is AFK.
         """
         activities = [x.to_dict() if not isinstance(x, dict) else x for x in activities]
-        return self.ws.update_presence(since, activities, status, afk)
+        if self.ws:
+            await self.ws.update_presence(since, activities, status, afk)
+        elif self.__shards:
+            for x in self.__shards.values():
+                await x.update_presence(since, activities, status, afk)
 
     def update_voice_state(self,
-                           guild: typing.Union[int, str, Snowflake, Guild],
-                           channel: typing.Union[int, str, Snowflake, Channel] = None,
+                           guild: Guild.TYPING,
+                           channel: Channel.TYPING = None,
                            self_mute: bool = False,
                            self_deaf: bool = False):
         """
@@ -183,26 +220,50 @@ class Client(APIClient):
         :param self_mute: Whether the bot is self-muted.
         :param self_deaf: Whether the bot is self-deaf.
         """
-        return self.ws.update_voice_state(str(int(guild)), str(int(channel)) if channel else None, self_mute, self_deaf)
+        if self.ws:
+            ws = self.ws
+        elif self.__shards:
+            ws = self.get_shard(guild)
+            if not ws:
+                raise AttributeError(f"shard for guild {guild.id} not found.")
+        else:
+            raise AttributeError(f"shard for guild {guild.id} not found.")
+        return ws.update_voice_state(str(int(guild)), str(int(channel)) if channel else None, self_mute, self_deaf)
 
     @property
-    def has_cache(self):
+    def has_cache(self) -> bool:
         return self.__use_cache
 
     @property
-    def websocket_closed(self):
+    def websocket_closed(self) -> typing.Union[bool, typing.List[bool]]:
         """
-        Whether the bot is disconnected from the Discord websocket.
-        :return: bool
+        Whether the bot is disconnected from the Discord websocket. If the bot is sharded, then it will return list of bools.
+        :return: Union[bool, List[bool]]
         """
         if self.ws:
             return self.ws.closed
+        elif self.__shards:
+            return [x.closed for x in self.__shards.values()]
         return True
 
     @property
-    def guild_count(self):
+    def guild_count(self) -> int:
         return self.cache.get_size("guild")
 
+    @property
+    def ping(self) -> float:
+        if self.ws:
+            return self.ws.ping
+        elif self.__shards:
+            pings = [x.ping for x in self.__shards.values()]
+            return sum(pings) / len(pings)
+        else:
+            return 0.0
+
+    @property
+    def shards(self) -> typing.Optional[typing.Tuple[WebSocketClient]]:
+        return tuple(self.__shards.values()) if self.__shards else None
+
     def __setattr__(self, key, value):
         if not key.lower().startswith("on_") or key.lower() in ["on", "on_"]:
             return super().__setattr__(key, value)
@@ -233,7 +294,8 @@ class Client(APIClient):
             This must be placed at the end of the code.
         """
         try:
-            self.loop.run_until_complete(self.start(reconnect_on_unknown_disconnect, compress))
+            self.loop.create_task(self.start(reconnect_on_unknown_disconnect, compress))
+            self.loop.run_forever()
         except KeyboardInterrupt:
             print("Detected KeyboardInterrupt, exiting...", file=sys.stderr)
         except Exception as ex:
