@@ -6,25 +6,25 @@ import asyncio
 import aiohttp
 from .ratelimit import RatelimitHandler
 from .. import exception, __version__
-from ..base.http import HTTPRequestBase, EmptyObject
+from ..base.http import HTTPRequestBase, EmptyObject, RESPONSE
 
 
 class AsyncHTTPRequest(HTTPRequestBase):
     def __init__(self,
                  token: str,
-                 loop: asyncio.AbstractEventLoop = None,
-                 session: aiohttp.ClientSession = None,
-                 default_retry: int = 3):
-        self.token = token.lstrip("Bot ")
-        self.logger = logging.getLogger("dico.http")
-        self.loop = loop or asyncio.get_event_loop()
-        self.session = session or aiohttp.ClientSession(loop=self.loop)
-        self.default_retry = default_retry
-        self._close_on_del = False
+                 loop: typing.Optional[asyncio.AbstractEventLoop] = None,
+                 session: typing.Optional[aiohttp.ClientSession] = None,
+                 default_retry: typing.Optional[int] = 3):
+        self.token: str = token.lstrip("Bot ")
+        self.logger: logging.Logger = logging.getLogger("dico.http")
+        self.loop: asyncio.AbstractEventLoop = loop or asyncio.get_event_loop()
+        self.session: aiohttp.ClientSession = session or aiohttp.ClientSession(loop=self.loop)
+        self.default_retry: int = default_retry
+        self._close_on_del: bool = False
         if not session:
             self._close_on_del = True
-        self._closed = False
-        self.ratelimits = RatelimitHandler()
+        self._closed: bool = False
+        self.ratelimits: RatelimitHandler = RatelimitHandler()
 
     def __del__(self):
         if not self._closed:
@@ -36,7 +36,7 @@ class AsyncHTTPRequest(HTTPRequestBase):
             await self.session.close()
         self._closed = True
 
-    async def request(self, route: str, meth: str, body: typing.Any = None, *, is_json: bool = False, reason_header: str = None, retry: int = None, **kwargs) -> dict:
+    async def request(self, route: str, meth: str, body: typing.Any = None, *, is_json: bool = False, reason_header: str = None, retry: int = None, **kwargs) -> RESPONSE:
         code = 429  # Empty code in case of rate limit fail.
         resp = {}   # Empty resp in case of rate limit fail.
         retry = (retry if retry > 0 else 1) if retry is not None else self.default_retry
@@ -103,8 +103,9 @@ class AsyncHTTPRequest(HTTPRequestBase):
                                   embeds: typing.List[dict] = None,
                                   allowed_mentions: dict = None,
                                   message_reference: dict = None,
-                                  components: typing.List[dict] = None):
-        if not (content or embeds or files):
+                                  components: typing.List[dict] = None,
+                                  sticker_ids: typing.List[str] = None) -> RESPONSE:
+        if not (content or embeds or files or sticker_ids):
             raise ValueError("either content or embed or files must be passed.")
         payload_json = {}
         form = aiohttp.FormData()
@@ -122,6 +123,8 @@ class AsyncHTTPRequest(HTTPRequestBase):
             payload_json["message_reference"] = message_reference
         if components:
             payload_json["components"] = components
+        if sticker_ids:
+            payload_json["sticker_ids"] = sticker_ids
         form.add_field("payload_json", json.dumps(payload_json), content_type="application/json")
         for x in range(len(files)):
             name = f"file{x if len(files) > 1 else ''}"
@@ -139,7 +142,7 @@ class AsyncHTTPRequest(HTTPRequestBase):
                                 files: typing.List[io.FileIO] = EmptyObject,
                                 allowed_mentions: dict = EmptyObject,
                                 attachments: typing.List[dict] = EmptyObject,
-                                components: typing.List[dict] = EmptyObject):
+                                components: typing.List[dict] = EmptyObject) -> RESPONSE:
         payload_json = {}
         form = aiohttp.FormData()
         if content is not EmptyObject:
@@ -175,7 +178,7 @@ class AsyncHTTPRequest(HTTPRequestBase):
                                    embeds: typing.List[dict] = None,
                                    allowed_mentions: dict = None,
                                    components: typing.List[dict] = None,
-                                   flags: int = None):
+                                   flags: int = None) -> RESPONSE:
         if not (content or embeds or files):
             raise ValueError("either content or embeds or files must be passed.")
         payload_json = {}
@@ -218,7 +221,7 @@ class AsyncHTTPRequest(HTTPRequestBase):
                              files: typing.List[io.FileIO] = EmptyObject,
                              allowed_mentions: dict = EmptyObject,
                              attachments: typing.List[dict] = EmptyObject,
-                             components: typing.List[dict] = EmptyObject):
+                             components: typing.List[dict] = EmptyObject) -> RESPONSE:
         payload_json = {}
         form = aiohttp.FormData()
         if content is not EmptyObject:
@@ -240,7 +243,7 @@ class AsyncHTTPRequest(HTTPRequestBase):
                 form.add_field(name, f, filename=sel.name, content_type="application/octet-stream")
         return self.request(f"/webhooks/{webhook_id}/{webhook_token}/messages/{message_id}", "PATCH", form)
 
-    async def download(self, url):
+    async def download(self, url) -> bytes:
         async with self.session.get(url) as resp:
             if resp.status == 200:
                 return await resp.read()
