@@ -1,5 +1,6 @@
 import asyncio
 import datetime
+import typing
 
 
 class __EmptyLocker:
@@ -15,9 +16,9 @@ EmptyLocker = __EmptyLocker()
 
 class RatelimitHandler:
     def __init__(self):
-        self.lockers = {}
-        self.buckets = {}
-        self.global_locker = asyncio.Lock()
+        self.lockers: typing.Dict[str, typing.Optional[dict]] = {}
+        self.buckets: dict = {}
+        self.global_locker: asyncio.Lock = asyncio.Lock()
 
     @staticmethod
     def to_locker_key(meth, route):
@@ -27,7 +28,7 @@ class RatelimitHandler:
     def utc(self):
         return datetime.datetime.utcnow()
 
-    def get_locker(self, meth, route):
+    def get_locker(self, meth, route) -> dict:
         locker_key = self.to_locker_key(meth, route)
         if locker_key not in self.lockers:
             self.lockers[locker_key] = None
