@@ -12,8 +12,9 @@ from .user import User
 from ..base.model import CopyableObject, DiscordObjectBase, TypeBase, FlagBase
 
 if typing.TYPE_CHECKING:
-    from .gateway import Application
+    from .application import Application
     from .guild import Guild
+    from .invite import Invite
     from ..api import APIClient
 
 
@@ -23,7 +24,7 @@ class Channel(DiscordObjectBase):
     RESPONSE_AS_LIST = typing.Union[typing.List["Channel"], typing.Awaitable[typing.List["Channel"]]]
     _cache_type = "channel"
 
-    def __init__(self, client, resp, *, guild_id=None):
+    def __init__(self, client: "APIClient", resp: dict, *, guild_id: Snowflake.TYPING = None):
         super().__init__(client, resp)
         self.type: ChannelTypes = ChannelTypes(resp["type"])
         self.guild_id: Snowflake = Snowflake.optional(resp.get("guild_id")) or Snowflake.ensure_snowflake(guild_id)
@@ -56,6 +57,9 @@ class Channel(DiscordObjectBase):
 
         # if self.type.dm and self.
 
+    def __str__(self) -> str:
+        return self.name
+
     def modify(self, **kwargs) -> RESPONSE:
         if self.type.group_dm:
             return self.client.modify_group_dm_channel(self.id, **kwargs)
@@ -73,7 +77,7 @@ class Channel(DiscordObjectBase):
     def delete(self, *, reason: str = None) -> RESPONSE:
         return self.client.delete_channel(self, reason=reason)
 
-    def create_message(self, *args, **kwargs) -> typing.Union["Message", typing.Awaitable["Message"]]:
+    def create_message(self, *args, **kwargs) -> "Message.RESPONSE":
         if not self.is_messageable():
             raise TypeError("You can't send message in this type of channel.")
         return self.client.create_message(self, *args, **kwargs)
@@ -88,22 +92,22 @@ class Channel(DiscordObjectBase):
     def edit_permissions(self, overwrite, *, reason: str = None):
         return self.client.edit_channel_permissions(self, overwrite, reason=reason)
 
-    def request_invites(self):
+    def request_invites(self) -> "Invite.RESPONSE_AS_LIST":
         return self.client.request_channel_invites(self)
 
-    def create_invite(self, **kwargs):
+    def create_invite(self, **kwargs) -> "Invite.RESPONSE":
         return self.client.create_channel_invite(self, **kwargs)
 
     def delete_permissions(self, overwrite, *, reason: str = None):
         return self.client.delete_channel_permission(self, overwrite, reason=reason)
 
-    def follow(self, target_channel: "Channel.TYPING"):
+    def follow(self, target_channel: "Channel.TYPING") -> "FollowedChannel.RESPONSE":
         return self.client.follow_news_channel(self, target_channel)
 
     def trigger_typing_indicator(self):
         return self.client.trigger_typing_indicator(self)
 
-    def request_pinned_messages(self):
+    def request_pinned_messages(self) -> "Message.RESPONSE_AS_LIST":
         return self.client.request_pinned_messages(self)
 
     def add_recipient(self, user: User.TYPING, access_token: str, nick: str):
@@ -116,7 +120,7 @@ class Channel(DiscordObjectBase):
             raise AttributeError("This type of channel is not allowed to remove recipient.")
         return self.client.group_dm_remove_recipient(self, user)
 
-    def start_thread(self, message: "Message.TYPING" = None, *, name: str, auto_archive_duration: int, reason: str = None):
+    def start_thread(self, message: "Message.TYPING" = None, *, name: str, auto_archive_duration: int, reason: str = None) -> "Channel.RESPONSE":
         return self.client.start_thread(self, message, name=name, auto_archive_duration=auto_archive_duration, reason=reason)
 
     def join_thread(self):
@@ -139,21 +143,21 @@ class Channel(DiscordObjectBase):
             raise AttributeError("This type of channel is not allowed to remove thread member.")
         return self.client.remove_thread_member(self, user)
 
-    def list_thread_members(self):
+    def list_thread_members(self) -> "ThreadMember.RESPONSE_AS_LIST":
         if not self.is_thread_channel():
             raise AttributeError("This type of channel is not allowed to list thread members.")
         return self.client.list_thread_members(self)
 
-    def list_active_threads(self):
+    def list_active_threads(self) -> "ListThreadsResponse.RESPONSE":
         return self.client.list_active_threads(self)
 
-    def list_public_archived_threads(self, *, before: typing.Union[str, datetime.datetime] = None, limit: int = None):
+    def list_public_archived_threads(self, *, before: typing.Union[str, datetime.datetime] = None, limit: int = None) -> "ListThreadsResponse.RESPONSE":
         return self.client.list_public_archived_threads(self, before=before, limit=limit)
 
-    def list_private_archived_threads(self, *, before: typing.Union[str, datetime.datetime] = None, limit: int = None):
+    def list_private_archived_threads(self, *, before: typing.Union[str, datetime.datetime] = None, limit: int = None) -> "ListThreadsResponse.RESPONSE":
         return self.client.list_private_archived_threads(self, before=before, limit=limit)
 
-    def list_joined_private_archived_threads(self, *, before: typing.Union[str, datetime.datetime] = None, limit: int = None):
+    def list_joined_private_archived_threads(self, *, before: typing.Union[str, datetime.datetime] = None, limit: int = None) -> "ListThreadsResponse.RESPONSE":
         return self.client.list_joined_private_archived_threads(self, before=before, limit=limit)
 
     def archive(self, locked: bool = False):
@@ -210,8 +214,8 @@ class VideoQualityModes(TypeBase):
 
 
 class SendOnlyChannel:
-    def __init__(self, client, channel_id):
-        self.client = client
+    def __init__(self, client: "APIClient", channel_id: Snowflake.TYPING):
+        self.client: "APIClient" = client
         self.id: Snowflake = Snowflake.ensure_snowflake(channel_id)
 
     def send(self, *args, **kwargs) -> "Message.RESPONSE":
@@ -227,7 +231,14 @@ class Message(DiscordObjectBase):
     RESPONSE_AS_LIST = typing.Union[typing.List["Message"], typing.Awaitable[typing.List["Message"]]]
     _cache_type = "message"
 
-    def __init__(self, client, resp, *, guild_id=None, webhook_token=None, interaction_token=None, original_response=False):
+    def __init__(self,
+                 client: "APIClient",
+                 resp: dict,
+                 *,
+                 guild_id: Snowflake.TYPING = None,
+                 webhook_token: typing.Optional[str] = None,
+                 interaction_token: typing.Optional[str] = None,
+                 original_response: typing.Optional[bool] = False):
         from .interactions import MessageInteraction, Component  # Prevent circular import.
         super().__init__(client, resp)
         self.channel_id: Snowflake = Snowflake(resp["channel_id"])
@@ -273,7 +284,10 @@ class Message(DiscordObjectBase):
 
         # self.stickers: typing.Optional[typing.List[MessageSticker]] = [MessageSticker(x) for x in resp.get("stickers", [])]
 
-    def reply(self, content=None, **kwargs) -> typing.Union["Message", typing.Awaitable["Message"]]:
+    def __str__(self) -> str:
+        return self.content
+
+    def reply(self, content=None, **kwargs) -> "Message.RESPONSE":
         kwargs["message_reference"] = self
         mention = kwargs.pop("mention") if "mention" in kwargs.keys() else True
         allowed_mentions = kwargs.get("allowed_mentions", self.client.default_allowed_mentions or AllowedMentions(replied_user=mention)).copy()
@@ -281,7 +295,7 @@ class Message(DiscordObjectBase):
         kwargs["allowed_mentions"] = allowed_mentions.to_dict(reply=True)
         return self.client.create_message(self.channel_id, content, **kwargs)
 
-    def edit(self, **kwargs):
+    def edit(self, **kwargs) -> "Message.RESPONSE":
         if self.__webhook_token:
             kwargs["webhook_token"] = self.__webhook_token
             return self.client.edit_webhook_message(self.webhook_id, self, **kwargs)
@@ -299,7 +313,7 @@ class Message(DiscordObjectBase):
             return self.client.delete_interaction_response(interaction_token=self.__interaction_token, message="@original" if self.__original_response else self)
         return self.client.delete_message(self.channel_id, self.id, reason=reason)
 
-    def crosspost(self):
+    def crosspost(self) -> "Message.RESPONSE":
         return self.client.crosspost_message(self.channel_id, self.id)
 
     def create_reaction(self, emoji: typing.Union[Emoji, str]):
@@ -314,7 +328,7 @@ class Message(DiscordObjectBase):
     def unpin(self, *, reason: str = None):
         return self.client.unpin_message(self.channel_id, self.id, reason=reason)
 
-    def start_thread(self, *, name: str, auto_archive_duration: int, reason: str = None):
+    def start_thread(self, *, name: str, auto_archive_duration: int, reason: str = None) -> Channel.RESPONSE:
         return self.client.start_thread(self.channel_id, self, name=name, auto_archive_duration=auto_archive_duration, reason=reason)
 
     @property
@@ -361,9 +375,9 @@ class MessageTypes(TypeBase):
 
 
 class MessageActivity:
-    def __init__(self, resp):
+    def __init__(self, resp: dict):
         self.type: MessageActivityTypes = MessageActivityTypes(resp["type"])
-        self.party_id: str = resp.get("party_id")  # This is actually set as string in discord docs.
+        self.party_id: typing.Optional[str] = resp.get("party_id")  # This is actually set as string in discord docs.
 
     @classmethod
     def optional(cls, resp):
@@ -390,7 +404,7 @@ class MessageFlags(FlagBase):
 
 
 class MessageReference:
-    def __init__(self, resp):
+    def __init__(self, resp: dict):
         self.message_id: typing.Optional[Snowflake] = Snowflake.optional(resp.get("message_id"))
         self.channel_id: typing.Optional[Snowflake] = Snowflake.optional(resp.get("channel_id"))
         self.guild_id: typing.Optional[Snowflake] = Snowflake.optional(resp.get("guild_id"))
@@ -419,7 +433,7 @@ class MessageReference:
 class FollowedChannel:
     RESPONSE = typing.Union["FollowedChannel", typing.Awaitable["FollowedChannel"]]
 
-    def __init__(self, client, resp):
+    def __init__(self, client: "APIClient", resp: dict):
         self.client: "APIClient" = client
         self.channel_id: typing.Optional[Snowflake] = Snowflake(resp["channel_id"])
         self.webhook_id: typing.Optional[Snowflake] = Snowflake(resp["webhook_id"])
@@ -431,7 +445,7 @@ class FollowedChannel:
 
 
 class Reaction:
-    def __init__(self, client, resp):
+    def __init__(self, client: "APIClient", resp: dict):
         self.count: int = resp["count"]
         self.me: bool = resp["me"]
         self.emoji: Emoji = Emoji(client, resp["emoji"])
@@ -472,12 +486,12 @@ class Overwrite(CopyableObject):
 
 
 class ThreadMetadata:
-    def __init__(self, client, resp):
-        self.client: APIClient = client
+    def __init__(self, client: "APIClient", resp: dict):
+        self.client: "APIClient" = client
         self.archived: bool = resp["archived"]
         # self.archiver_id: typing.Optional[Snowflake] = Snowflake.optional(resp.get("archiver_id"))
         self.auto_archive_duration: int = resp["auto_archive_duration"]
-        self.archive_timestamp = datetime.datetime.fromisoformat(resp["archive_timestamp"])
+        self.archive_timestamp: datetime.datetime = datetime.datetime.fromisoformat(resp["archive_timestamp"])
         self.locked: bool = resp["locked"]
         self.invitable: typing.Optional[bool] = resp.get("invitable")
 
@@ -491,8 +505,8 @@ class ThreadMember:
     RESPONSE = typing.Union["ThreadMember", typing.Awaitable["ThreadMember"]]
     RESPONSE_AS_LIST = typing.Union[typing.List["ThreadMember"], typing.Awaitable[typing.List["ThreadMember"]]]
 
-    def __init__(self, client, resp):
-        self.client: APIClient = client
+    def __init__(self, client: "APIClient", resp: dict):
+        self.client: "APIClient" = client
         self.id: typing.Optional[Snowflake] = Snowflake.optional(resp.get("id"))
         self.user_id: typing.Optional[Snowflake] = Snowflake.optional(resp.get("user_id"))
         self.join_timestamp: datetime.datetime = datetime.datetime.fromisoformat(resp["join_timestamp"])
@@ -596,7 +610,7 @@ class Embed(CopyableObject):
 
 
 class EmbedThumbnail(CopyableObject):
-    def __init__(self, resp):
+    def __init__(self, resp: dict):
         self.url: typing.Optional[str] = resp.get("url")
         self.proxy_url: typing.Optional[str] = resp.get("proxy_url")
         self.height: typing.Optional[int] = resp.get("height")
@@ -621,7 +635,7 @@ class EmbedThumbnail(CopyableObject):
 
 
 class EmbedVideo(CopyableObject):
-    def __init__(self, resp):
+    def __init__(self, resp: dict):
         self.url: typing.Optional[str] = resp.get("url")
         self.proxy_url: typing.Optional[str] = resp.get("proxy_url")
         self.height: typing.Optional[int] = resp.get("height")
@@ -646,7 +660,7 @@ class EmbedVideo(CopyableObject):
 
 
 class EmbedImage(CopyableObject):
-    def __init__(self, resp):
+    def __init__(self, resp: dict):
         self.url: typing.Optional[str] = resp.get("url")
         self.proxy_url: typing.Optional[str] = resp.get("proxy_url")
         self.height: typing.Optional[int] = resp.get("height")
@@ -671,7 +685,7 @@ class EmbedImage(CopyableObject):
 
 
 class EmbedProvider(CopyableObject):
-    def __init__(self, resp):
+    def __init__(self, resp: dict):
         self.name: typing.Optional[str] = resp.get("name")
         self.url: typing.Optional[str] = resp.get("url")
 
@@ -690,7 +704,7 @@ class EmbedProvider(CopyableObject):
 
 
 class EmbedAuthor(CopyableObject):
-    def __init__(self, resp):
+    def __init__(self, resp: dict):
         self.name: typing.Optional[str] = resp.get("name")
         self.url: typing.Optional[str] = resp.get("url")
         self.icon_url: typing.Optional[str] = resp.get("icon_url")
@@ -715,7 +729,7 @@ class EmbedAuthor(CopyableObject):
 
 
 class EmbedFooter(CopyableObject):
-    def __init__(self, resp):
+    def __init__(self, resp: dict):
         self.text: typing.Optional[str] = resp["text"]
         self.icon_url: typing.Optional[str] = resp.get("icon_url")
         self.proxy_icon_url: typing.Optional[str] = resp.get("proxy_icon_url")
@@ -735,7 +749,7 @@ class EmbedFooter(CopyableObject):
 
 
 class EmbedField(CopyableObject):
-    def __init__(self, resp):
+    def __init__(self, resp: dict):
         self.name: typing.Optional[str] = resp["name"]
         self.value: typing.Optional[str] = resp["value"]
         self.inline: typing.Optional[bool] = resp.get("inline", True)
@@ -750,8 +764,8 @@ class EmbedField(CopyableObject):
 
 
 class Attachment:
-    def __init__(self, client, resp):
-        self.client: APIClient = client
+    def __init__(self, client: "APIClient", resp: dict):
+        self.client: "APIClient" = client
         self.id: Snowflake = Snowflake(resp["id"])
         self.filename: str = resp["filename"]
         self.content_type: typing.Optional[str] = resp.get("content_type")
@@ -761,6 +775,7 @@ class Attachment:
         self.height: typing.Optional[int] = resp.get("height")
         self.width: typing.Optional[int] = resp.get("width")
         self.content: typing.Optional[bytes] = None  # Filled after download is called
+        self.ephemeral: typing.Optional[bool] = resp.get("ephemeral")
 
     def download(self) -> typing.Union[bytes, typing.Awaitable[bytes]]:
         dw = self.client.http.download(self.url)
@@ -795,7 +810,7 @@ class Attachment:
 
 
 class ChannelMention:
-    def __init__(self, resp):
+    def __init__(self, resp: dict):
         self.id: Snowflake = Snowflake(resp["id"])
         self.guild_id: Snowflake = Snowflake(resp["guild_id"])
         self.type: ChannelTypes = ChannelTypes(resp["type"])
@@ -811,7 +826,7 @@ class AllowedMentions(CopyableObject):
         self.everyone: bool = everyone
         self.users: typing.List[Snowflake.TYPING] = users or []
         self.roles: typing.List[Snowflake.TYPING] = roles or []
-        self.replied_user = replied_user
+        self.replied_user: bool = replied_user
 
     def to_dict(self, *, reply: bool = False) -> dict:
         ret = {"parsed": []}
@@ -831,7 +846,7 @@ class AllowedMentions(CopyableObject):
 class ListThreadsResponse:
     RESPONSE = typing.Union["ListThreadsResponse", typing.Awaitable["ListThreadsResponse"]]
 
-    def __init__(self, client, resp):
+    def __init__(self, client: "APIClient", resp: dict):
         self.threads: typing.List[Channel] = [Channel.create(client, x) for x in resp["threads"]]
         self.members: typing.List[ThreadMember] = [ThreadMember(client, x) for x in resp["members"]]
         self.has_more: typing.Optional[bool] = resp.get("has_more")
