@@ -4,6 +4,7 @@ from .snowflake import Snowflake
 
 if typing.TYPE_CHECKING:
     from .permission import Role
+    from ..api import APIClient
 
 
 class Emoji:
@@ -11,7 +12,7 @@ class Emoji:
     RESPONSE = typing.Union["Emoji", typing.Awaitable["Emoji"]]
     RESPONSE_AS_LIST = typing.Union[typing.List["Emoji"], typing.Awaitable[typing.List["Emoji"]]]
 
-    def __init__(self, client, resp):
+    def __init__(self, client: "APIClient", resp: dict):
         self.id: typing.Optional[Snowflake] = Snowflake.optional(resp.get("id"))
         self.name: str = resp["name"]
         self.roles: typing.Optional[typing.List["Role"]] = [client.get(x) for x in resp.get("roles", [])] if client.has_cache else [Snowflake.optional(x) for x in resp.get("roles", [])]
@@ -22,8 +23,8 @@ class Emoji:
         self.animated: typing.Optional[bool] = resp.get("animated")
         self.available: typing.Optional[bool] = resp.get("available")
 
-    def __str__(self):
+    def __str__(self) -> str:
         return f"<:{self.name}:{self.id}>" if self.id else self.name
 
-    def __int__(self):
+    def __int__(self) -> int:
         return int(self.id)
