@@ -5,13 +5,13 @@ from ..exception import WebsocketRateLimited
 
 
 class WSRatelimit:
-    def __init__(self, heartbeat_time=6):
-        self.count = 0
-        self.init_time = time.time()
-        self.locker = asyncio.Lock()
-        self.max_requests = int(120 - (60 / heartbeat_time))
+    def __init__(self, heartbeat_time: int = 6):
+        self.count: int = 0
+        self.init_time: float = time.time()
+        self.locker: asyncio.Lock = asyncio.Lock()
+        self.max_requests: int = int(120 - (60 / heartbeat_time))
 
-    def maybe_limited(self):
+    def maybe_limited(self) -> asyncio.Lock:
         now = time.time()
         if self.init_time + 60 >= now and self.count >= self.max_requests:
             raise WebsocketRateLimited((self.init_time + 60) - now)
@@ -24,5 +24,5 @@ class WSRatelimit:
     def reload_heartbeat(self, heartbeat_time):
         self.max_requests = int(120 - (60 / heartbeat_time))
 
-    def reset_after(self):
+    def reset_after(self) -> float:
         return self.init_time + 60 - time.time()
